@@ -4,7 +4,7 @@
 Protocols relating to molecular biology, e.g. PCR.
 """
 
-__version__ = '1.22.0'
+__version__ = '1.23.0'
 
 from ._utils import *
 from ._assembly import Assembly
@@ -23,6 +23,8 @@ from .ivt import Ivt
 from .ivtt import InVitroTranslation
 from .kld import Kld
 from .ligate import Ligate
+from .lyophilize import Lyophilize
+from .miniprep import Miniprep
 from .pcr import Pcr
 from .serial_dilution import SerialDilution
 from .spin_cleanup import SpinCleanup
@@ -32,10 +34,13 @@ from .invpcr import InversePcr
 from .qpcr import Qpcr
 from .page_purify import PagePurify
 
+import stepwise
 from pathlib import Path
 
 class Plugin:
     protocol_dir = Path(__file__).parent
-    config_defaults = protocol_dir / 'conf.toml'
+    config_path = protocol_dir / 'conf.toml'
+    priority = stepwise.Builtins.priority + 10
 
+del stepwise
 del Path
