@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 import parametrize_from_file
-from schema_helpers import *
+from param_helpers import *
 
 @parametrize_from_file(schema=app_expected_protocol)
 def test_protocol(app, expected):
